#include "my_timer.h"
#define TABLE_NAME_LENGTH 21
#define SUF ".rbt"



typedef struct {
       int key_size, hash_size;             // values in bits
       int table_size;                      // number of pairs [key, hash]
                                            // if upgrading table, this value means number of pairs to be
                                            // added to the exitsting table
       int chain_length;                    // number of pairs [key, hash]
       int array_size;                      // size of array to be allocated in memory; 
                                            // value in number of pairs [key, hash] 
                                            // i.e. size_in_bytes = ((key_size+hash_size)/8) * array_size
       int remove_duplicates;               // if not zero, duplicates are being removed 
                                            // while merging blocks
       int strict_dupl_detection;           // if not zero, procedure createTable will consider
                                            // chains with same endings as collisions
       int lookups_on;                      // if non zero, function upgrade will try to detect collision with
                                            // existing table immediately after creating chain
       int lookups;                         // max number of chains thrown away while adding chains 
                                            // used in situation if lookupsOn is true
       
       int limit1;                          // max nuber of attempts to replace chains removed due to collisions
       int limit2;                                
       char table_name[TABLE_NAME_LENGTH];  // identifies the project
                                            // program creates files: 
                                            // [table_name].rbt [table_name]2.rbt ([table_name]SUF, [table_name]2SUF)
                                            // [table_name]1.tmp, [table_name]1a.tmp,
                                            // [table_name]2.tmp, [table_name]2a.tmp
       
       void (*fEncode)(void* key, void* hash);
       void (*fReduce)(void* hash, int i, void* key);
       int (*fCompare)(const void* hashkey1,const void* hashkey2);
       void (*fGenerateKey) (void* key);
       
       // OUTPUT VARIABLES
       // CREATING TABLE 
       int collisions;                      // number of collisions detected while creating table
       int chains;                          // number of chains created
       my_time create_time;                  // time of creating table
       my_time create_chains_time;          // time of creating chains while creating table
       my_time upgrade_time;                // time of upgrading table
       my_time upgrade_chains_time;         // analogue
       // SEARCHING IN TABLE
       int row;                             // position of item that was found in table 
                                            // if no item was found, valaue is undefined
       int column;                          // number of column of table (chain) where the key was found
                                            // 1..chain_length       
       void *pKey;                            // points on the key tah was found in table
                                            // if no item was found, valaue is undefined                                            
                                            
       int false_alarms;                    // number of false alarms during searching
       int nencode;                         // number of calls of encode function
       int false_nencode;                   // number of calls of encode function during false alarms
       int nreduce, false_nreduce;          // analogue
       int ncompare;                        // nuber of compare calls
       
       my_time search_time;
       
} TableInfo;

// creates table in file ([ti->tablename].rbt)
// for chain of length n, the reduce function is called with values of i: 1, 2, ... 
int createTable(TableInfo *ti);
    
// searches the table stored in ([ti->tablename].rbt)
// *item is pointer on hash value ( function adds zeroes in front of the value to be 
// compatibile with compare function )
// if table contains the key we want, funcition returns 1 else returns 0;
int searchTable(TableInfo *ti, const void *item);

// adds ti->table_size chains to the existing table identified by ti->project_name
// parameters key_size, hash_size and chain_length are ignored, function loads values
// from <ti->project_name>2.rbt
int upgradeTable(TableInfo *ti);

//
int createDescriptionFile(const TableInfo *ti);

// saves values of variables in *ti into *filename;
void saveToTextfile(const TableInfo *ti, char *filename);

void saveToTextfile2(const TableInfo *ti, char *filename, char *open_type, void *item);

// loads key_size, hash_size, table_size and chain_length
// table of errors:
// 0 - allOK   
// 1 - table does not exists
// 2 - description file does not exists
// 3 - neither of the files exists
int loadMainParameters(TableInfo *ti, int loadTableSize);

